@@ -1,6 +1,6 @@
 #  You may distribute under the terms of the GNU General Public License
 #
-#  (C) Paul Evans, 2012-2013 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2012-2015 -- leonerd@leonerd.org.uk
 
 package Circle::FE::Term::Tab;
 
@@ -9,7 +9,7 @@ use warnings;
 
 use Tickit::Widget::Tabbed;
 use base qw( Tickit::Widget::Tabbed::Tab );
-Tickit::Widget::Tabbed->VERSION( '0.008' );
+Tickit::Widget::Tabbed->VERSION( '0.019' );
 
 use Tickit::Term 0.27; # setctl_str
 
@@ -30,53 +30,77 @@ sub new
    my $object = delete $args{object};
    my $self;
 
+   my $watch_tag;
    if( $object->proxy_isa( "Circle.RootObj" ) ) {
       $args{label} = "Global";
    }
    else {
       $args{label} = $object->prop( "tag" );
-      $object->watch_property(
-         property => "tag",
-         on_set => sub {
-            my ( $newtag ) = @_;
-            $self->set_label_text( $newtag );
-         },
-      );
+      $watch_tag++;
    }
 
    $self = $class->SUPER::new( $tabbed, %args );
    $self->{object} = $object;
    $self->{term} = $tabbed->window->term;
 
-   $object->call_method(
-      method => "get_widget",
-      args => [],
-      on_result => sub {
+   $self->adopt_future(
+      $object->call_method(
+         get_widget => (),
+      )->on_done( sub {
          $self->widget->add( $self->build_widget( $_[0] ), expand => 1 );
-      }
+      })
    );
 
-   $object->watch_property(
-      property => "level",
-      on_set => sub {
-         my ( $level ) = @_;
-         $self->set_level( $level );
-      },
-      want_initial => 1,
+   $self->adopt_future(
+      $object->watch_property_with_initial(
+         "level",
+         on_set => sub {
+            my ( $level ) = @_;
+            $self->set_level( $level );
+         },
+      )
    );
 
-   $object->subscribe_event(
-      event => "raise",
-      on_fire => sub {
-         $self->activate;
-      },
+   $self->adopt_future(
+      $object->subscribe_event(
+         "raise",
+         on_fire => sub {
+            $self->activate;
+         },
+      )
    );
 
+   if( $watch_tag ) {
+      $self->adopt_future(
+         $object->watch_property(
+            "tag",
+            on_set => sub {
+               my ( $newtag ) = @_;
+               $self->set_label_text( $newtag );
+            },
+         )
+      );
+   }
+
    $self->set_on_activated( 'activated' );
 
    return $self;
 }
 
+sub adopt_future
+{
+   my $self = shift;
+   my ( $f ) = @_;
+
+   my $futures = $self->{futures} //= {};
+
+   $futures->{"$f"} = $f;
+
+   $f->on_ready( sub {
+      delete $futures->{shift()};
+   });
+}
+
 sub build_widget
 {
    my $self = shift;
@@ -101,7 +125,7 @@ sub set_level
    my $self = shift;
    my ( $level ) = @_;
 
-   $self->pen->chattr( fg => Circle::FE::Term->get_theme_colour( "level$level" ) );
+   $self->set_pen( Circle::FE::Term->get_theme_pen( "level$level" ) );
 }
 
 sub set_label_text
@@ -136,10 +160,10 @@ sub activated
    my $object = $self->{object};
 
    if( $object->prop("level") > 0 ) {
-      $object->call_method(
-         method => "reset_level",
-         args   => [],
-         on_result => sub {}, # ignore
+      $self->adopt_future(
+         $object->call_method(
+            reset_level => (),
+         )
       );
    }
 
